﻿using System;

namespace Unclassified.UI
{
	/// <summary>
	/// Sets the mouse cursor of a view to the wait cursor.
	/// </summary>
	class ViewWaitCursor : IDisposable
	{
		private ISupportsWaitCursor view;

		/// <summary>
		/// Sets the cursor to the wait cursor until this instance is disposed.
		/// </summary>
		/// <param name="view">View to set the cursor on.</param>
		public ViewWaitCursor(ISupportsWaitCursor view)
		{
			this.view = view;
			view.IsWaitCursor = true;
		}

		/// <summary>
		/// Resets the cursor to its default value.
		/// </summary>
		public void Dispose()
		{
			view.IsWaitCursor = false;
		}
	}

	/// <summary>
	/// Sets the mouse cursor of a view to the background wait cursor.
	/// </summary>
	class ViewBackgroundWaitCursor : IDisposable
	{
		private ISupportsBackgroundWaitCursor view;

		/// <summary>
		/// Sets the cursor to the background wait cursor until this instance is disposed.
		/// </summary>
		/// <param name="view">View to set the cursor on.</param>
		public ViewBackgroundWaitCursor(ISupportsBackgroundWaitCursor view)
		{
			this.view = view;
			view.IsBackgroundWaitCursor = true;
		}

		/// <summary>
		/// Resets the cursor to its default value.
		/// </summary>
		public void Dispose()
		{
			view.IsBackgroundWaitCursor = false;
		}
	}

	/// <summary>
	/// Provides the IsWaitCursor property to set an element's wait cursor.
	/// </summary>
	interface ISupportsWaitCursor
	{
		/// <summary>
		/// Gets or sets a value indicating whether the wait cursor is active.
		/// </summary>
		bool IsWaitCursor { get; set; }
	}

	/// <summary>
	/// Provides the IsBackgroundWaitCursor property to set an element's wait cursor.
	/// </summary>
	interface ISupportsBackgroundWaitCursor
	{
		/// <summary>
		/// Gets or sets a value indicating whether the background wait cursor is active.
		/// </summary>
		bool IsBackgroundWaitCursor { get; set; }
	}
}
