﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Threading;
using System.Xml;
using CassiniSwitch.Model;
using Unclassified.UI;

namespace CassiniSwitch.ViewModel
{
	class MainViewModel : ViewModelBase, ISupportsWaitCursor
	{
		#region Private fields

		private Window view;
		private CassiniInstaller installer;

		#endregion Private fields

		#region Constructor

		public MainViewModel(Window view)
		{
			this.view = view;
			DisplayName = "CassiniSwitch";

			installer = new CassiniInstaller();
		}

		#endregion Constructor

		#region Public properties

		#endregion Public properties

		#region View control

		private void AllPropertiesChanged()
		{
			OnPropertyChanged("IsVS2008Available");
			OnPropertyChanged("IsVS2008X64Available");
			OnPropertyChanged("IsVS2008DevServer");
			OnPropertyChanged("IsVS2008Cassini");
			OnPropertyChanged("IsVS2008X64");
			OnPropertyChanged("VS2008Version");

			OnPropertyChanged("IsVS2010Available");
			OnPropertyChanged("IsVS2010X64Available");
			OnPropertyChanged("IsVS2010DevServer");
			OnPropertyChanged("IsVS2010Cassini");
			OnPropertyChanged("IsVS2010X64");
			OnPropertyChanged("VS2010Version");

			OnPropertyChanged("IsVS2012Available");
			OnPropertyChanged("IsVS2012X64Available");
			OnPropertyChanged("IsVS2012DevServer");
			OnPropertyChanged("IsVS2012Cassini");
			OnPropertyChanged("IsVS2012X64");
			OnPropertyChanged("VS2012Version");
		}

		#endregion View control

		#region Commands

		private DelegateCommand openInstallDirCommand;
		public DelegateCommand OpenInstallDirCommand
		{
			get
			{
				if (openInstallDirCommand == null)
				{
					openInstallDirCommand = new DelegateCommand(() =>
					{
						try
						{
							Process.Start(installer.GetInstallDirectory(0));
						}
						catch (Exception ex)
						{
							installer.ReportError("The installation base directory could not be opened.", ex);
						}
					});
				}
				return openInstallDirCommand;
			}
		}

		private DelegateCommand openAppWebsiteCommand;
		public DelegateCommand OpenAppWebsiteCommand
		{
			get
			{
				if (openAppWebsiteCommand == null)
				{
					openAppWebsiteCommand = new DelegateCommand(() =>
					{
						try
						{
							Process.Start("http://dev.unclassified.de/apps/cassiniswitch");
						}
						catch (Exception ex)
						{
							installer.ReportError("The website could not be opened.", ex);
						}
					});
				}
				return openAppWebsiteCommand;
			}
		}

		private DelegateCommand openCassiniWebsiteCommand;
		public DelegateCommand OpenCassiniWebsiteCommand
		{
			get
			{
				if (openCassiniWebsiteCommand == null)
				{
					openCassiniWebsiteCommand = new DelegateCommand(() =>
					{
						try
						{
							Process.Start("https://cassinidev.codeplex.com/");
						}
						catch (Exception ex)
						{
							installer.ReportError("The website could not be opened.", ex);
						}
					});
				}
				return openCassiniWebsiteCommand;
			}
		}

		#endregion Commands

		#region Data properties

		private Cursor cursor;
		public Cursor Cursor
		{
			get { return cursor; }
			set
			{
				if (value != cursor)
				{
					cursor = value;
					OnPropertyChanged("Cursor");
				}
			}
		}

		/// <summary>
		/// Gets or sets a value indicating whether the wait cursor is active.
		/// </summary>
		public bool IsWaitCursor
		{
			get { return Cursor == Cursors.Wait; }
			set { Cursor = value ? Cursors.Wait : null; }
		}

		public string MyVersion
		{
			get
			{
				return CassiniInstaller.AssemblyFileVersion.ToString();
			}
		}

		public string MyCopyright
		{
			get
			{
				return CassiniInstaller.AssemblyCopyright.ToString();
			}
		}

		#region VS 2008

		public bool IsVS2008Available
		{
			get
			{
				return installer.IsVS2008Available;
			}
		}

		public bool IsVS2008X64Available
		{
			get
			{
				return IsVS2008Available && IsVS2008Cassini && Environment.Is64BitOperatingSystem;
			}
		}

		public bool IsVS2008DevServer
		{
			get
			{
				return installer.IsVS2008DevServer;
			}
			set
			{
				if (value != IsVS2008DevServer)
				{
					using (new ViewWaitCursor(this))
					{
						installer.UninstallCassini(2008);
						AllPropertiesChanged();
					}
				}
			}
		}

		public bool IsVS2008Cassini
		{
			get
			{
				return installer.IsVS2008Cassini;
			}
			set
			{
				if (value != IsVS2008Cassini)
				{
					using (new ViewWaitCursor(this))
					{
						installer.InstallCassini(2008, installer.IsVS2008X64);
						AllPropertiesChanged();
					}
				}
			}
		}

		public bool IsVS2008X64
		{
			get
			{
				return installer.IsVS2008X64;
			}
			set
			{
				if (value != IsVS2008X64)
				{
					using (new ViewWaitCursor(this))
					{
						installer.InstallCassini(2008, value);
						AllPropertiesChanged();
					}
				}
			}
		}

		public string VS2008Version
		{
			get
			{
				if (!installer.IsVS2008Available)
				{
					return "Not available.";
				}
				if (installer.IsVS2008DevServer)
				{
					return "WebDev server installed.";
				}
				if (installer.VS2008Version != null)
				{
					return "Cassini " + installer.VS2008Version + " installed.";
				}
				return "Not installed.";
			}
		}

		#endregion VS 2008

		#region VS 2010

		public bool IsVS2010Available
		{
			get
			{
				return installer.IsVS2010Available;
			}
		}

		public bool IsVS2010X64Available
		{
			get
			{
				return IsVS2010Available && IsVS2010Cassini && Environment.Is64BitOperatingSystem;
			}
		}

		public bool IsVS2010DevServer
		{
			get
			{
				return installer.IsVS2010DevServer;
			}
			set
			{
				if (value != IsVS2010DevServer)
				{
					using (new ViewWaitCursor(this))
					{
						installer.UninstallCassini(2010);
						AllPropertiesChanged();
					}
				}
			}
		}

		public bool IsVS2010Cassini
		{
			get
			{
				return installer.IsVS2010Cassini;
			}
			set
			{
				if (value != IsVS2010Cassini)
				{
					using (new ViewWaitCursor(this))
					{
						installer.InstallCassini(2010, installer.IsVS2010X64);
						AllPropertiesChanged();
					}
				}
			}
		}

		public bool IsVS2010X64
		{
			get
			{
				return installer.IsVS2010X64;
			}
			set
			{
				if (value != IsVS2010X64)
				{
					using (new ViewWaitCursor(this))
					{
						installer.InstallCassini(2010, value);
						AllPropertiesChanged();
					}
				}
			}
		}

		public string VS2010Version
		{
			get
			{
				if (!installer.IsVS2010Available)
				{
					return "Not available.";
				}
				if (installer.IsVS2010DevServer)
				{
					return "WebDev server installed.";
				}
				if (installer.VS2010Version != null)
				{
					return "Cassini " + installer.VS2010Version + " installed.";
				}
				return "Not installed.";
			}
		}

		#endregion VS 2010

		#region VS 2012

		public bool IsVS2012Available
		{
			get
			{
				return installer.IsVS2012Available;
			}
		}

		public bool IsVS2012X64Available
		{
			get
			{
				return IsVS2012Available && IsVS2012Cassini && Environment.Is64BitOperatingSystem;
			}
		}

		public bool IsVS2012DevServer
		{
			get
			{
				return installer.IsVS2012DevServer;
			}
			set
			{
				if (value != IsVS2012DevServer)
				{
					using (new ViewWaitCursor(this))
					{
						installer.UninstallCassini(2012);
						AllPropertiesChanged();
					}
				}
			}
		}

		public bool IsVS2012Cassini
		{
			get
			{
				return installer.IsVS2012Cassini;
			}
			set
			{
				if (value != IsVS2012Cassini)
				{
					using (new ViewWaitCursor(this))
					{
						installer.InstallCassini(2012, installer.IsVS2012X64);
						AllPropertiesChanged();
					}
				}
			}
		}

		public bool IsVS2012X64
		{
			get
			{
				return installer.IsVS2012X64;
			}
			set
			{
				if (value != IsVS2012X64)
				{
					using (new ViewWaitCursor(this))
					{
						installer.InstallCassini(2012, value);
						AllPropertiesChanged();
					}
				}
			}
		}

		public string VS2012Version
		{
			get
			{
				if (!installer.IsVS2012Available)
				{
					return "Not available.";
				}
				if (installer.IsVS2012DevServer)
				{
					return "WebDev server installed.";
				}
				if (installer.VS2012Version != null)
				{
					return "Cassini " + installer.VS2012Version + " installed.";
				}
				return "Not installed.";
			}
		}

		#endregion VS 2012

		#endregion Data properties

		#region Command handlers

		#endregion Command handlers
	}
}
